﻿# -*- coding: utf-8 -*-

import bpy
from bpy.props import BoolProperty

if(1):
	import importlib
	loc = locals()
	if("ui" in locals()):
		importlib.reload(ui)
	if("op" in locals()):
		importlib.reload(op)
	del loc

from . import ui
from . import op

#### =======================================================================
def _Regist(*arg):
	Fn = bpy.utils.register_class
	for ar in arg:
		for cls in reversed(ar):
			if(cls):
				Fn(cls)

def _Unregist(*arg):
	Fn = bpy.utils.unregister_class
	for ar in arg:
		for cls in ar:
			if(cls):
				Fn(cls)

#### =======================================================================
# unregistする固有classの退避用
_gOBJECT_PT_visibility = None
_gVIEW3D_PT_tools_grease_pencil_brush = None
_gVIEW3D_PT_tools_brush = None

#### =======================================================================
def PostRegister():
	'''registの後処理'''
	pref = bpy.context.preferences.addons["AsBldTools"].preferences
	types = bpy.types
	if(pref.useDelUnusedObjs):
		types.TOPBAR_MT_edit.append(op.DeleteUnusedObjects.AddMenu)
	if(pref.useNodeToCtxMenu):
		types.NODE_MT_context_menu.prepend(ui.NodeToCtxMenu.AddMenu)
	if(pref.usePropertiesHeadTab):
		types.PROPERTIES_HT_header.append(ui.PropertiesHeader.Menu_header)
	if(pref.useConsoleFontSize):
		types.CONSOLE_HT_header.append(ui.ConsoleHeader.AddMenu)
	if(pref.useMakeBlankShape):
		types.VIEW3D_MT_mesh_add.append(op.Object_MakeBlankShape.AddMenu)
	if(pref.useCameraZoom):
		types.VIEW3D_PT_view3d_lock.append(ui.View3dSideBar.Menu_view3d_lock)
	if(pref.useRemoveHierarchies):
		types.VIEW3D_MT_object.append(op.Object_RemoveHierarchies.AddMenu)

	# unregistする固有classの退避
	global _gOBJECT_PT_visibility
	global _gVIEW3D_PT_tools_brush
	global _gVIEW3D_PT_tools_grease_pencil_brush
	_gOBJECT_PT_visibility = (bpy.types.OBJECT_PT_visibility,)
	_gVIEW3D_PT_tools_grease_pencil_brush = (bpy.types.VIEW3D_PT_tools_grease_pencil_brush,)
	_gVIEW3D_PT_tools_brush = [					# 順に注意
		bpy.types.VIEW3D_PT_tools_brush_options,
		bpy.types.VIEW3D_PT_tools_brush_clone,
		None,
		bpy.types.VIEW3D_PT_tools_brush_swatches,
		bpy.types.VIEW3D_PT_tools_brush_color,
		bpy.types.VIEW3D_PT_tools_brush,
		bpy.types.IMAGE_PT_paint_options,
		bpy.types.IMAGE_PT_paint_clone,
		None,
		bpy.types.IMAGE_PT_paint_swatches,
		bpy.types.IMAGE_PT_paint_color,
		bpy.types.IMAGE_PT_paint,
		]
	if(bpy.app.version[1] == 80):
		_gVIEW3D_PT_tools_brush[2] = bpy.types.VIEW3D_PT_tools_brush_gradient
		_gVIEW3D_PT_tools_brush[8] = bpy.types.IMAGE_PT_paint_gradient

	if(pref.hidePropertiesVisPanels):
		_Unregist(_gOBJECT_PT_visibility)

	if(pref.hideBrushPanels):
		_Unregist(_gVIEW3D_PT_tools_brush, _gVIEW3D_PT_tools_grease_pencil_brush)

#### --------------------------------
def PreUnregister():
	'''unregistの前処理'''
	pref = bpy.context.preferences.addons["AsBldTools"].preferences
	types = bpy.types
	if(pref.useDelUnusedObjs):
		types.TOPBAR_MT_edit.remove(op.DeleteUnusedObjects.AddMenu)
	if(pref.useNodeToCtxMenu):
		types.NODE_MT_context_menu.remove(ui.NodeToCtxMenu.AddMenu)
	if(pref.usePropertiesHeadTab):
		types.PROPERTIES_HT_header.remove(ui.PropertiesHeader.Menu_header)
	if(pref.useConsoleFontSize):
		types.CONSOLE_HT_header.remove(ui.ConsoleHeader.AddMenu)
	if(pref.useMakeBlankShape):
		types.VIEW3D_MT_mesh_add.remove(op.Object_MakeBlankShape.AddMenu)
	if(pref.useCameraZoom):
		types.VIEW3D_PT_view3d_lock.remove(ui.View3dSideBar.Menu_view3d_lock)
	if(pref.useRemoveHierarchies):
		types.VIEW3D_MT_object.remove(op.Object_RemoveHierarchies.AddMenu)

	if(pref.hidePropertiesVisPanels):	# 無効化してたら復元
		global _gOBJECT_PT_visibility
		_Regist(_gOBJECT_PT_visibility)

	if(pref.hideBrushPanels):			# 無効化してたら復元
		global _gVIEW3D_PT_tools_brush
		global _gVIEW3D_PT_tools_grease_pencil_brush
		_Regist(_gVIEW3D_PT_tools_brush, _gVIEW3D_PT_tools_grease_pencil_brush)

#### =======================================================================
class AsBldToolsPrefs(bpy.types.AddonPreferences):
	'''アドオン設定'''
	bl_idname = "AsBldTools"

	#### ----------------
	def Update_DelUnusedObjs(self, context):
		'''未使用オブジェクトを削除'''
		menu = bpy.types.TOPBAR_MT_edit
		Fn = menu.append if(self.useDelUnusedObjs) else menu.remove
		Fn(op.DeleteUnusedObjects.AddMenu)

	#### ----------------
	def Update_NodeToCtxMenu(self, context):
		'''ノードコンテキストメニューにノード作成'''
		menu = bpy.types.NODE_MT_context_menu
		Fn = menu.prepend if(self.useNodeToCtxMenu) else menu.remove
		Fn(ui.NodeToCtxMenu.AddMenu)

	#### ----------------
	def Update_NodeToSideBar(self, context):
		'''ノード作成をサイドバーに追加'''
		if(self.useNodeToSideBar):
			ui.NodeToSideBar.register()
		else:
			ui.NodeToSideBar.unregister()

	#### ----------------
	def Update_PropertiesHeadTab(self, context):
		'''プロパティヘッダにポップアップ/タブ追加'''
		header = bpy.types.PROPERTIES_HT_header
		Fn = header.append if(self.usePropertiesHeadTab) else header.remove
		Fn(ui.PropertiesHeader.Menu_header)

	#### ----------------
	def Update_ConsoleFontSize(self, context):
		'''コンソールヘッダのフォントサイズ'''
		header = bpy.types.CONSOLE_HT_header
		Fn = header.append if(self.useConsoleFontSize) else header.remove
		Fn(ui.ConsoleHeader.AddMenu)

	#### ----------------
	def Update_MakeBlankShape(self, context):
		'''空のシェイプ'''
		menu = bpy.types.VIEW3D_MT_mesh_add
		Fn = menu.append if(self.useMakeBlankShape) else menu.remove
		Fn(op.Object_MakeBlankShape.AddMenu)

	#### ----------------
	def Update_CameraZoom(self, context):
		'''カメラズーム'''
		menu = bpy.types.VIEW3D_PT_view3d_lock
		Fn = menu.append if(self.useCameraZoom) else menu.remove
		Fn(ui.View3dSideBar.Menu_view3d_lock)

	#### ----------------
	def Update_RemoveHierarchies(self, context):
		'''階層削除'''
		menu = bpy.types.VIEW3D_MT_object
		Fn = menu.append if(self.useRemoveHierarchies) else menu.remove
		Fn(op.Object_RemoveHierarchies.AddMenu)

	#### ----------------
	def Update_PropertiesVisPanels(self, context):
		'''プロパティパネルから可視性PTを除去'''
		global _gOBJECT_PT_visibility
		if(self.hidePropertiesVisPanels):
			_Unregist(_gOBJECT_PT_visibility)
		else:
			_Regist(_gOBJECT_PT_visibility)

	#### ----------------
	def Update_BrushListSelectors(self, context):
		'''リスト型ブラシ選択'''
		if(self.useBrushListSelectors):
			ui.PaintBrushSelector.register()
			ui.GPencilListSelector.register()
		else:
			ui.PaintBrushSelector.unregister()
			ui.GPencilListSelector.unregister()

	#### ----------------
	def Update_BrushPanels(self, context):
		'''プレビュー方ブラシ選択を無効化'''
		global _gVIEW3D_PT_tools_brush
		global _gVIEW3D_PT_tools_grease_pencil_brush
		if(self.hideBrushPanels):
			_Unregist(_gVIEW3D_PT_tools_brush, _gVIEW3D_PT_tools_grease_pencil_brush)
		else:
			_Regist(_gVIEW3D_PT_tools_brush, _gVIEW3D_PT_tools_grease_pencil_brush)

	#### ----------------
	useDelUnusedObjs: BoolProperty(name= "未使用オブジェクトを全て削除", default= True,
		description= "TopBarのEditMenuに「未使用オブジェクトを全て削除」\nを追加(object.delete_unused_objects).",
		update= Update_DelUnusedObjs,)

	useNodeToCtxMenu: BoolProperty(name= "CtxMenuにノード作成を追加", default= True,
		description= "NodeEditorのContextMenuにノード追加項目を追加.",
		update= Update_NodeToCtxMenu,)

	useNodeToSideBar: BoolProperty(name= "SideBarにノードタブを追加", default= True,
		description= "NodoEditorのSideBarにノード追加タブを追加.",
		update= Update_NodeToSideBar,)

	usePropertiesHeadTab: BoolProperty(name= "プロパティーヘッダにタブを追加", default= True,
		description= "プロパティーパネルのヘッダに切替タブを追加.",
		update= Update_PropertiesHeadTab,)

	useConsoleFontSize: BoolProperty(name= "コンソールヘッダにFontSizeを追加", default= True,
		description= "PythonコンソールのヘッダにFontSizeプロパティを追加.",
		update= Update_ConsoleFontSize,)

	useMakeBlankShape: BoolProperty(name= "空シェイプを作成", default= True,
		description= "View3DのAddMenu→Meshに「ブランク」\nを追加(object.make_blank_shape).",
		update= Update_MakeBlankShape,)

	useCameraZoom: BoolProperty(name= "カメラズーム/オフセットを追加", default= True,
		description= "View3DのSideBar→ビュー→ビューのロックに\nカメラズームとオブセットプロパティを追加.\nカメラをビューにロックOn/Offに依存しません.",
		update= Update_CameraZoom,)

	useRemoveHierarchies: BoolProperty(name= "オブジェクトを階層ごと削除", default= True,
		description= "View3DのObjectMenuに「オブジェクトを階層ごと削除」\nを追加(object.remove_hierarchies).",
		update= Update_CameraZoom,)

	useOutlinerSelectBox: BoolProperty(name= "アウトライナの矩形選択と実選択(2.80向け)", default= True,
		description= "アウトライナ用の矩形選択と実選択処理(outliner.select_box_with_select).",)

	useObjectGrouping: BoolProperty(name= "Emptyでグループ化", default= True,
		description= "Emptyでグループ化(object.grouping).",)

	hidePropertiesVisPanels: BoolProperty(name= "プロパティーの可視性枠を無効化", default= False,
		description= "プロパティパネルの可視性枠を無効化.",
		update= Update_PropertiesVisPanels,)

	useBrushListSelectors: BoolProperty(name= "リスト型ブラシ選択を追加", default= True,
		description= "ブラシ選択をリストから行うブラシ枠を追加.",
		update= Update_BrushListSelectors,)

	hideBrushPanels: BoolProperty(name= "プレビュー型ブラシ選択を無効化", default= False,
		description= "ブラシ(プレビュー)枠を無効化.",
		update= Update_BrushPanels,)

	#### ----------------
	keyNames = (
		("Node", ("useNodeToCtxMenu", "useNodeToSideBar")),
		("Operator",
			("useDelUnusedObjs", "useMakeBlankShape", "useObjectGrouping", "useRemoveHierarchies",
			"useOutlinerSelectBox")
			),
		("UI", ("usePropertiesHeadTab", "useBrushListSelectors", "useCameraZoom", "useConsoleFontSize",
			"hidePropertiesVisPanels","hideBrushPanels")),
		)

	#### ----------------
	def hoge(self, base, label, keyNames):
		clm = base.column()
		clm.label(text=label)
		box = clm.box()
		flw = box.grid_flow(columns=2)
		for k in keyNames:
			flw.prop(self, k)

	#### ----------------
	def draw(self, context):
		baseBox = self.layout.box()
		baseBox.scale_y = .9

		for (l, k) in self.keyNames:
			self.hoge(baseBox, l, k)
