﻿# -*- coding: utf-8 -*-

import bpy

#### =======================================================================
class OBJECT_OT_DeleteUnusedObjects(bpy.types.Operator):
	'''未使用オブジェクトを削除. 消す事で未使用になる物も削除.'''
	bl_idname = "object.delete_unused_objects"
	bl_label  = "未使用オブジェクトを全て削除"
	bl_options= {"REGISTER", "UNDO"}

	def execute(self, context):
		# module層やclass層でbpy.data.*は取れない,method内で毎回取るしか無い.
		dt = bpy.data
		entries = (		# 消して行く順序は重要
			dt.meshes,
			dt.curves,
			dt.armatures,
			dt.materials,
			dt.images,
			dt.movieclips,
			dt.lights,
			dt.cameras,
			dt.objects,
		)
		res = []
		EXT = res.extend
		for dat in entries:
			ob = tuple( o for o in dat.values() if((not o.use_fake_user) and (not o.users)) )
			if(ob):
				EXT([ o.name for o in ob ])
				for o in ob:
					dat.remove(o)

		sks = tuple( o for o in bpy.data.shape_keys if((not o.use_fake_user) and (not o.users)) )
		if(sks):
			EXT([ o.name for o in sks ])
			bpy.data.batch_remove(ids= sks)

		if(res):
			self.report({"OPERATOR"}, ",".join(res))

		return({"FINISHED"})

#### =======================================================================
def AddMenu(self, context):
	'''TOPBAR_MT_edit 追加用'''
	self.layout.separator()
	self.layout.operator(OBJECT_OT_DeleteUnusedObjects.bl_idname, text= OBJECT_OT_DeleteUnusedObjects.bl_label, icon="ORPHAN_DATA")
