﻿# -*- coding: utf-8 -*-

import bpy
import gpu
from gpu.types import GPUBatch, GPUVertBuf, GPUVertFormat

#### =======================================================================
# batch(type)参考 tkengo.github.io/blog/2015/01/03/opengl-es-2-2d-knowledge-2/
# LINE系にalpha指定は効かない？
# サンプルに倣ってmodule(global)上でshader取得(独自vshader/fshader)等をすると,
# blender終了時にメモリ未開放のログが出る. builtinだと出ないぽい

#### =======================================================================
class OUTLINER_OT_SelectBoxWithSelect(bpy.types.Operator):
	'''矩形選択と実選択処理'''
	bl_idname = "outliner.select_box_with_select"
	bl_label  = "矩形選択と実選択処理"
	bl_options= {"GRAB_CURSOR"}

	use_cursor_region = True

	xmin: bpy.props.IntProperty(name="X Min", default=0)
	xmax: bpy.props.IntProperty(name="X Max", default=0)
	ymin: bpy.props.IntProperty(name="Y Min", default=0)
	ymax: bpy.props.IntProperty(name="Y Max", default=0)
	wait_for_input: bpy.props.BoolProperty(name="Wait for input", default=False)
	mode: bpy.props.EnumProperty(name="Mode", default="SET",
		items=(
			("SET", "Set", "Set a new selection.", "SELECT_SET", 0),
			("ADD", "Extend", "Extend existing selection.", "SELECT_EXTEND", 1),
			("SUB", "Subtract", "Subtract existing selection.", "SELECT_SUBTRACT", 2),
		))

	#### --------------------------------
	def MakeShader(self):
#		shader = gpu.shader.from_builtin("2D_FLAT_COLOR")
#		shader = gpu.shader.from_builtin("2D_UNIFORM_COLOR")
#		vtFmt  = shader.format_calc()

		vs = "uniform mat4 ModelViewProjectionMatrix; in vec2 pos;" \
			 "void main(){gl_Position = ModelViewProjectionMatrix * vec4(pos, 0f, 1f);}"
		fs = "void main(){gl_FragColor=vec4(1f, 1f, 1f, 1f);}"
		shader = gpu.types.GPUShader(vs, fs)
		vtFmt = GPUVertFormat()
		vtFmt.attr_add(id="pos", comp_type="F32", len=2, fetch_mode="FLOAT")

		return(shader, vtFmt)

	#### --------------------------------
	def Shader_Draw(self, context):
		vtPos = ((self.xmin, self.ymin), (self.xmin, self.ymax), (self.xmax, self.ymax), (self.xmax, self.ymin),)
		vbo = GPUVertBuf(self.vtFmt, 4)	# len(vtPos)
		vbo.attr_fill("pos", vtPos)

		# 2D_FLAT_COLOR
#		vbo.attr_fill("color", ((1.0, 1.0, 1.0, 1.0),)*len(vtPos))

		# 2D_UNIFORM_COLOR
#		self.shader.bind()
#		self.shader.uniform_float("color", (1.0, 1.0, 1.0, 1.0))

		# 非builtin, colorはflagShaderでハードコード

		GPUBatch(type= "LINE_LOOP", buf= vbo).draw(self.shader)

	#### --------------------------------
	def __init__(self):
		(self.shader, self.vtFmt) = self.MakeShader()

	#### --------------------------------
	def __del__(self):
		gpu.shader.unbind()	# お呪い, 有効かは不明

	#### --------------------------------
	@classmethod
	def poll(cls, context):
		return(context.space_data.type == "OUTLINER")

	#### --------------------------------
	def modal(self, context, event):
		context.area.tag_redraw()
		if(event.value == "PRESS"):
			self.xmin = min(self.sx, event.mouse_region_x)
			self.xmax = max(self.sx, event.mouse_region_x)
			self.ymin = min(self.sy, event.mouse_region_y)
			self.ymax = max(self.sy, event.mouse_region_y)

		elif(event.value == "RELEASE"):
			bpy.types.SpaceOutliner.draw_handler_remove(self.hdl, "WINDOW")
			return(self.execute(context))

		return({"RUNNING_MODAL"})

	#### --------------------------------
	def invoke(self, context, event):
		if(event.type in { "EVT_TWEAK_L", "EVT_TWEAK_M", "EVT_TWEAK_R" }):
			context.window_manager.modal_handler_add(self)
			self.sx = event.mouse_region_x
			self.sy = event.mouse_region_y
			self.hdl = bpy.types.SpaceOutliner.draw_handler_add(self.Shader_Draw, (context,), "WINDOW", "POST_PIXEL")
			return({"RUNNING_MODAL"})
		return({"PASS_THROUGH"})

	#### --------------------------------
	def execute(self, context):
		bpy.ops.outliner.select_box(wait_for_input= self.wait_for_input, mode= self.mode,
			xmin= self.xmin,
			xmax= self.xmax,
			ymin= self.ymin,
			ymax= self.ymax,
			)
		if(self.mode != "ADD"):
			bpy.ops.object.select_all(action="DESELECT")	# outliner.select_allでは不可, 行マーカーもクリアされる.
		bpy.ops.outliner.object_operation(type="SELECT")

		return({"FINISHED"})
