﻿# -*- coding: utf-8 -*-

import bpy
import bl_ui.properties_paint_common as ppc
from bl_ui.space_view3d_toolbar import VIEW3D_PT_tools_brush
from bpy.app.handlers import persistent

from .BrushSelector_Sub import *

#### =======================================================================
_v3dInst = None
_imgInst = None

_gToolNameDct = {}
for (k, v) in ( ("Draw","DRAW"),("Soften","SOFTEN"),("Smear","SMEAR"),("Clone","CLONE"),("Fill","FILL"),("Mask","MASK"),("Blur","BLUR"),
				("Average","AVERAGE"),("Clay","CLAY"),("Clay Strips","CLAY_STRIPS"),("Layer","LAYER"),("Inflate","INFLATE"),("Blob","BLOB"),
				("Crease","CREASE"),("Smooth","SMOOTH"),("Flatten","FLATTEN"),("Scrape","SCRAPE"),("Pinch","PINCH"),("Grab","GRAB"),
				("Snake Hook","SNAKE_HOOK"),("Thumb","THUMB"),("Nudge","NUDGE"),("Rotate","ROTATE"),("Simplify","SIMPLIFY"),("Mask","MASK"),
				("Paint","PAINT"),("Orco Draw","ORCO"),("Relax","RELAX")):	# paint/orco/relaxはsculptFuture
	_gToolNameDct["builtin_brush.{}".format(k)] = v

for (k, v) in (("gradient","gradient"),("sample_weight","sample_weight"),("box_mask","box_mask"),("box_hide","box_hide"),("annotate","annotate")):
	_gToolNameDct["builtin.{}".format(k)] = v

_gModeSet = {"SCULPT", "PAINT_VERTEX", "PAINT_WEIGHT", "PAINT_TEXTURE"}
_gModeAttrDct = {
	"SCULPT":"sculpt", "PAINT_VERTEX":"vertex_paint", "PAINT_WEIGHT":"weight_paint",
	"PAINT_TEXTURE":"image_paint", "PAINT":"image_paint",
	}
_gNonBrushToolIdSet = {"builtin.gradient", "builtin.sample_weight", "builtin.box_mask", "builtin.box_hide", "builtin.annotate"}

_gIgnoreModeSet = {"EDIT_MESH","EDIT_CURVE","EDIT_SURFACE","EDIT_TEXT","EDIT_ARMATURE","EDIT_METABALL","EDIT_LATTICE","POSE","PAINT_GPENCIL","EDIT_GPENCIL","SCULPT_GPENCIL","WEIGHT_GPENCIL" }

#### =======================================================================
#### -----------------------------------------------------------------------
@persistent
def _UpdateDep_V3d(scene):
	'''tool切替には反応しない. 実行時はcontext.space_dataはNone.
	GUIでpreview選択(選択変化問わず)→updateDep.(SCENE: dirty, BRUSH: clean)→prop→Dep
	GUIでリスト操作(idx変化問わず)→updateProp→brush更新→updateDep.(SCENE: dirty, BRUSH: clean)
	GUIでの操作は同じ要素を選んでもSCENEがdirtyになる.
	ブラシ追加()→updateDep.(SCENE: dirty, BRUSH: dirty)
	(NG: dep()内で.brushを変更すると循環に入る)'''
	context = bpy.context
	mode = context.mode

	if(mode in _gIgnoreModeSet):
		return

	if((mode not in _gModeSet) or (mode == "OBJECT")):	# 直接modeが切り替わらずにOBJECTが挟まってくる
		_v3dInst.SetDirty(context.workspace.tools.from_space_view3d_mode(mode).idname)
		_v3dInst.Flush()
		return

	if(_v3dInst.isDirty):
		brush = getattr(scene.tool_settings, _gModeAttrDct[mode]).brush
		if(brush):
			FindBrushIndex(scene, brush.name, "PaintBrushSelector_V3d_Index")
		_v3dInst.Flush()
		return

	if(not _v3dInst.isCutoffDep):	# prop→depの中断,dep→prop→depの再入防止.
		SearchBrush(scene, context, getattr(scene.tool_settings, _gModeAttrDct[mode]).brush, "PaintBrushSelector_V3d_Index")
	_v3dInst.Flush()

#### -----------------------------------------------------------------------
@persistent
def _UpdateDep_Img(scene):
	'''実行時はcontext.space_dataが無い為, 打ち切りの基準点を持てない.
	imageEditはcontext.image_paint_objectに依らない'''
	context = bpy.context

	if(_imgInst.isDirty):
		brush = context.tool_settings.image_paint.brush
		if(brush):
			FindBrushIndex(scene, brush.name, "PaintBrushSelector_Img_Index")
		_imgInst.Flush()
		return

	if(not _imgInst.isCutoffDep):	# prop→depの中断,dep→prop→depの再入防止.
		SearchBrush(scene, context, context.tool_settings.image_paint.brush, "PaintBrushSelector_Img_Index")
	_imgInst.Flush()

#### =======================================================================
def _UpdateProp_V3d(scene, context):
	return(UpdateProp(scene, context, _v3dInst, ppc.UnifiedPaintPanel.paint_settings(context), scene.PaintBrushSelector_V3d_Index))

#### -----------------------------------------------------------------------
def _UpdateProp_Img(scene, context):
	return(UpdateProp(scene, context, _imgInst, context.tool_settings.image_paint, scene.PaintBrushSelector_Img_Index))

#### =======================================================================
class VIEW3D_UL_PaintBrushSelector(bpy.types.UIList, UL_DrawFuncs):
	''' '''
	# updateDepを起動する為のpropとupdateFunc, スコープはclass内(非インスタンス)縛り？
	forceUpdate: bpy.props.BoolProperty(default=True, options={"HIDDEN","SKIP_SAVE"}, update= lambda scn, ctx: None)

	def filter_items(self, context, data, propName):
		'''flag True==0x40000000, False==0'''
		mode = context.mode
		curToolId = context.workspace.tools.from_space_view3d_mode(mode).idname

		if(_v3dInst.toolId != curToolId):
			_v3dInst.SetDirty(curToolId)
			self.forceUpdate = True	# 値はなんでもいい

		#アクティブブラシが無くなる仕様バグへの対処として.brushのチェックが必要
		if((not getattr(context.tool_settings, _gModeAttrDct[mode]).brush) or (curToolId in _gNonBrushToolIdSet)):
			return([], [])

		toolName = _gToolNameDct[curToolId]
		if(context.sculpt_object):
			filter_flags = [ (int(brush.use_paint_sculpt and (brush.sculpt_tool == toolName)) << 30) for brush in data.brushes.values() ]

		elif(context.vertex_paint_object):
			filter_flags = [ (int(brush.use_paint_vertex and (brush.vertex_tool == toolName)) << 30) for brush in data.brushes.values() ]

		elif(context.weight_paint_object):
			filter_flags = [ (int(brush.use_paint_weight and (brush.weight_tool == toolName)) << 30) for brush in data.brushes.values() ]

		elif(context.image_paint_object and context.tool_settings.image_paint.detect_data()):
			filter_flags = [ (int(brush.use_paint_image and (brush.image_tool == toolName)) << 30) for brush in data.brushes.values() ]

		else:
			return([], [])

		return(filter_flags, [])

#### =======================================================================
class IMAGE_UL_PaintBrushSelector(bpy.types.UIList, UL_DrawFuncs):
	''' '''
	# updateDepを起動する為のpropとupdateFunc, スコープはclass内(非インスタンス)縛り？
	forceUpdate: bpy.props.BoolProperty(default=True, options={"HIDDEN","SKIP_SAVE"}, update= lambda scn, ctx: None)

	def filter_items(self, context, data, propName):
		'''flag True==0x40000000, False==0'''
		tool = context.workspace.tools.from_space_image_mode(context.space_data.mode)
		if(not tool):	# 未初期化バグでツール未選択が起き得る
			return([], [])

		curToolId = tool.idname

		if(_imgInst.toolId != curToolId):
			_imgInst.SetDirty(curToolId)
			self.forceUpdate = True	# 値はなんでもいい

		#アクティブブラシが無くなる仕様バグへの対処として.brushのチェックが必要
		if((not context.tool_settings.image_paint.brush) or (curToolId in _gNonBrushToolIdSet)):
			return([], [])

		# imageEditは context.image_paint_objectとdetect_data()に依らない
		toolName = _gToolNameDct[curToolId]
		filter_flags = [ (int(brush.use_paint_image and (brush.image_tool == toolName)) << 30) for brush in data.brushes.values() ]

		return(filter_flags, [])

#### =======================================================================
def _Draw_HeadLabel(layout, brush):
	layout.alignment = "RIGHT"
	layout.label(text= "({})".format(brush.name) if(brush) else "()", translate=False)

#### =======================================================================
class VIEW3D_PT_PaintBrushSelector(bpy.types.Panel, ppc.UnifiedPaintPanel):
	''' '''
	bl_space_type = "VIEW_3D"
	bl_region_type = "UI"
	bl_category = "Tool"
	bl_context = ".paint_common"
	bl_label = "Brush"

	#### --------------------------------
	poll = VIEW3D_PT_tools_brush.poll

	#### --------------------------------
	def draw_header_preset(self, context):
		_Draw_HeadLabel(self.layout, getattr(context.tool_settings, _gModeAttrDct[context.mode]).brush)

	#### --------------------------------
	def draw(self, context):
		'''self==UnifiedPaintPanel, template_list.maxrowsは効かない?'''
		layout = self.layout
		layout.use_property_split = True

		# context.tool_settings.sculpt/vertex_paint/weight_paint/image_paint/particle_edit
		settings = self.paint_settings(context)
		if(not settings):
			return

		brush = settings.brush
		isBrushTool = (_gToolNameDct[context.workspace.tools.from_space_view3d_mode(context.mode).idname] not in _gNonBrushToolIdSet)

		clm = layout.column()
		clm.use_property_split = False

		if(not self.is_popover):
			row = clm.row(align=True)
			row.template_list("VIEW3D_UL_PaintBrushSelector", "", context.blend_data, "brushes", context.scene, "PaintBrushSelector_V3d_Index", rows=1)
			row.enabled = isBrushTool and bool(brush)
			# アクティブブラシが無くなる仕様バグへの対処.
			# None時にリスト選択してもプレビュ側のpyがbrush.gpencil_settingsでエラーを吐く.
			# 共用ブラシではブラシ関係のPTが非表示になる(ツール再選択で復活)

			clm = row.column()
			clm.operator("brush.add", text="", icon="DUPLICATE")

			if(brush):
				clm.prop(brush, "use_fake_user", text="", icon_only=True)
			else:
				clm.label(icon="FAKE_USER_ON")

		if((not isBrushTool) or (not brush)):
			return

		col = layout.column()

		if(context.sculpt_object):
			if(not self.is_popover):
				ppc.brush_basic_sculpt_settings(col, context, brush)

			# normal_radius_factor
			col.prop(brush, "normal_radius_factor", slider=True)
			col.prop(brush, "curve_preset")
			col.prop(brush, "automasking_mode")
			col.prop(brush, "sculpt_color_mix_mode")

			cap = brush.sculpt_capabilities
			# topology_rake_factor
			if(cap.has_topology_rake and context.sculpt_object.use_dynamic_topology_sculpting):
				col.prop(brush, "topology_rake_factor", slider=True)

			# auto_smooth_factor and use_inverse_smooth_pressure
			if(cap.has_auto_smooth):
				row = col.row(align=True)
				row.prop(brush, "auto_smooth_factor", slider=True)
				row.prop(brush, "use_inverse_smooth_pressure", toggle=True, text="")

			if(cap.has_normal_weight):
				col.prop(brush, "normal_weight", slider=True)

			if(cap.has_pinch_factor):
				col.prop(brush, "crease_pinch_factor", slider=True, text="Pinch")

			if(cap.has_rake_factor):
				col.prop(brush, "rake_factor", slider=True)

			if(brush.sculpt_tool == "MASK"):
				col.prop(brush, "mask_tool")

			# plane_offset, use_offset_pressure, use_plane_trim, plane_trim
			if(cap.has_plane_offset):
				row = col.row(align=True)
				row.prop(brush, "plane_offset", slider=True)
				row.prop(brush, "use_offset_pressure", text="")

				col.prop(brush, "use_plane_trim", text="Plane Trim")

				row = col.row()
				row.active = brush.use_plane_trim
				row.prop(brush, "plane_trim", slider=True, text="Distance")

			if(cap.has_height):
				col.prop(brush, "height", slider=True, text="Height")

			# use_persistent, set_persistent_base
			if(cap.has_persistence):
				# not supported yet for this case
				for md in context.sculpt_object.modifiers:
					if(md.type == 'MULTIRES'):
						break
				else:
					col.prop(brush, "use_persistent")
					col.operator("sculpt.set_persistent_base")

		elif(context.image_paint_object):
			ppc.brush_texpaint_common(self, context, col, brush, settings, True)

		elif(context.weight_paint_object and (not self.is_popover)):
			ppc.brush_basic_wpaint_settings(col, context, brush)

		elif(context.vertex_paint_object and (not self.is_popover)):
			ppc.brush_basic_vpaint_settings(col, context, brush)

#### =======================================================================
class IMAGE_PT_PaintBrushSelector(bpy.types.Panel, ppc.UnifiedPaintPanel):
	''' '''
	bl_space_type = "IMAGE_EDITOR"
	bl_region_type = "UI"
	bl_category = "Tool"
	bl_context = ".paint_common_2d"
	bl_label = "Brush"

	#### --------------------------------
	@classmethod
	def poll(cls, context):
		# imgEditと画像指定で縛る
		return((context.space_data.type == "IMAGE_EDITOR") and bool(context.space_data.image))

	#### --------------------------------
	def draw_header_preset(self, context):
		_Draw_HeadLabel(self.layout, context.tool_settings.image_paint.brush)

	#### --------------------------------
	def draw(self, context):
		'''self==UnifiedPaintPanel, template_list.maxrowsは効かない?'''
		layout = self.layout
		layout.use_property_split = True

		clm = layout.column()
		clm.use_property_split = False

		row = clm.row(align=True)
		row.template_list("IMAGE_UL_PaintBrushSelector", "", context.blend_data, "brushes", context.scene, "PaintBrushSelector_Img_Index", rows=1)

		# 有効なimg指定が無い状態や操作順によってツール選択がされないバグを踏んだ時はtools.from()の戻りはNone
		tool = context.workspace.tools.from_space_image_mode(context.space_data.mode)
		if(tool):
			isBrushTool = (_gToolNameDct[tool.idname] not in _gNonBrushToolIdSet)
		else:
			isBrushTool = False

		brush = context.tool_settings.image_paint.brush

		row.enabled = isBrushTool and bool(brush)

		clm = row.column()
		clm.operator("brush.add", text="", icon="DUPLICATE")

		if(brush):
			clm.prop(brush, "use_fake_user", text="", icon_only=True)
			ppc.brush_texpaint_common(self, context, layout, brush, context.tool_settings.image_paint)
		else:
			clm.label(icon="FAKE_USER_ON")

#### =======================================================================
_clss = [
	VIEW3D_UL_PaintBrushSelector,
	VIEW3D_PT_PaintBrushSelector,
	VIEW3D_PT_PaintBrushSelector_tools_brush_color,
	VIEW3D_PT_PaintBrushSelector_tools_brush_swatches,
	None,
	VIEW3D_PT_PaintBrushSelector_tools_brush_clone,
	VIEW3D_PT_PaintBrushSelector_tools_brush_options,

	IMAGE_UL_PaintBrushSelector,
	IMAGE_PT_PaintBrushSelector,
	IMAGE_PT_PaintBrushSelector_paint_color,
	IMAGE_PT_PaintBrushSelector_paint_swatches,
	None,
	IMAGE_PT_PaintBrushSelector_paint_clone,
	IMAGE_PT_PaintBrushSelector_paint_options,
	]
if(bpy.app.version[1] == 80):
	_clss[4] = VIEW3D_PT_PaintBrushSelector_tools_brush_gradient
	_clss[10]= IMAGE_PT_PaintBrushSelector_paint_gradient

#### =======================================================================
def register():
	global _v3dInst
	global _imgInst
	_v3dInst = FlagData()
	_imgInst = FlagData()

	bpy.types.Scene.PaintBrushSelector_V3d_Index = bpy.props.IntProperty(default=0, min=0, subtype="UNSIGNED", options={"HIDDEN","SKIP_SAVE"}, update=_UpdateProp_V3d)
	bpy.types.Scene.PaintBrushSelector_Img_Index = bpy.props.IntProperty(default=0, min=0, subtype="UNSIGNED", options={"HIDDEN","SKIP_SAVE"}, update=_UpdateProp_Img)

	bpy.app.handlers.depsgraph_update_post.append(_UpdateDep_V3d)
	bpy.app.handlers.depsgraph_update_post.append(_UpdateDep_Img)
	Fn = bpy.utils.register_class
	for cls in _clss:
		if(cls):
			Fn(cls)

#### -----------------------------------------------------------------------
def unregister():
	Fn = bpy.utils.unregister_class
	for cls in _clss:
		if(cls):
			Fn(cls)
	bpy.app.handlers.depsgraph_update_post.remove(_UpdateDep_Img)
	bpy.app.handlers.depsgraph_update_post.remove(_UpdateDep_V3d)

	del bpy.types.Scene.PaintBrushSelector_Img_Index
	del bpy.types.Scene.PaintBrushSelector_V3d_Index

	global _v3dInst
	global _imgInst
	_v3dInst = None
	_imgInst = None
